@@ -3,7 +3,7 @@ package Assert::Refute::Report;
 use 5.006;
 use strict;
 use warnings;
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 
 =head1 NAME
 
@@ -33,6 +33,13 @@ See L<Assert::Refute::Contract> for contract I<definition>.
 
 =cut
 
+# Now this module is the CORE of Assert::Refute.
+# There are 3 things for which performance matters:
+# 1) new()
+# 2) refute( 0, ... )
+# 3) done_testing()
+# The rest can wait.
+
 use Carp;
 use Scalar::Util qw(blessed);
 
@@ -71,6 +78,42 @@ sub new {
 
 =head2 RUNNING PRIMITIVES
 
+=head3 plan( tests => n )
+
+Plan to run exactly n tests.
+This is not required, and L<done_testing> (see below)
+is needed at the end anyway.
+
+Dies if there's already a plan, or tests are being run, or done_testing
+was seen.
+
+If plan is not fullfilled by the time of C<done_testing> call,
+a message indicating plan violation will be added,
+and the report will unconditionally failing.
+
+=cut
+
+sub plan {
+    my ($self, $todo, @args) = @_;
+
+    $self->_croak( $ERROR_DONE )
+        if $self->{done};
+
+    if ($todo eq 'tests') {
+        $self->_croak( "Usage: plan tests => n")
+            unless @args == 1 and defined $args[0] and $args[0] =~ /^\d+$/;
+        $self->_croak( "Plan already defined" )
+            if defined $self->{plan_tests};
+        $self->_croak( "Testing already started" )
+            if $self->{count} > 0;
+        $self->{plan_tests} = $args[0];
+    } else {
+        $self->_croak( "Unknown 'plan $todo ...' command" );
+    };
+
+    return $self;
+};
+
 =head3 refute( $condition, $message )
 
 An inverted assertion. That is, it B<passes> if C<$condition> is B<false>.
@@ -85,18 +128,21 @@ See L<Assert::Refute/refute> for more detailed discussion.
 sub refute {
     my ($self, $cond, $msg) = @_;
 
-    $msg = $msg ? " - $msg" : '';
-    my $n = ++$self->{count};
+    $self->_croak( $ERROR_DONE )
+        if $self->{done};
 
-    if ($cond) {
-        $self->set_result( $n, $cond );
-        $self->do_log( 0, -2, "not ok $n$msg" );
-        $self->do_log( 0, -1, $cond ) unless $cond eq 1;
-        return 0;
-    } else {
-        $self->do_log( 0,  0, "ok $n$msg" );
-        return 1;
-    };
+    my $n = ++$self->{count};
+    $self->{name}{$n} = $msg if defined $msg;
+    delete $self->{log}; # log is a shortcut to $self->{messages}{$n}
+                         # see do_log()
+
+    # Pass, return ASAP
+    return $n unless $cond;
+
+    # Test failed!
+    $self->{fail}{$n} = $cond;
+    $self->{fail_count}++;
+    return 0;
 };
 
 =head3 diag
@@ -153,19 +199,31 @@ sub done_testing {
     my ($self, $exception) = @_;
 
     if ($exception) {
+        # Record a totally failing contract.
         delete $self->{done};
         $self->{has_error} = $exception;
-        # Make sure there *is* a failing test on the outside
-        $self->refute( $exception, "unexpected exception: $exception" );
-        $self->do_log( 0, 1, "Looks like test execution was interrupted" );
+        $self->diag( "Looks like contract was interrupted by", $exception );
     } elsif ($self->{done}) {
+        # A special case - done_testing(0) means "tentative stop"
         return $self if defined $exception;
         $self->_croak( $ERROR_DONE );
-    } else {
-        $self->do_log(0, 0, "1..$self->{count}");
     };
-    $self->do_log(0, 1,
-        "Looks like $self->{fail_count} tests of $self->{count} have failed")
+
+    # Any post-mortem messages go to a separate bucket
+    $self->{log} = $self->{messages}{ $self->{count}+1 } = [];
+
+    if (defined $self->{plan_tests}) {
+        # Check plan
+        if ($self->{count} != $self->{plan_tests}) {
+            my $bad_plan = "Looks like you planned $self->{plan_tests}"
+                ." tests but ran $self->{count}";
+            $self->{has_error} ||= $bad_plan;
+            $self->diag( $bad_plan );
+        };
+    };
+
+    $self->diag(
+        "Looks like $self->{fail_count} tests out of $self->{count} have failed")
             if $self->{fail_count};
 
     $self->{done}++;
@@ -216,6 +274,11 @@ test conditions, and is required.
 sub subcontract {
     my ($self, $msg, $sub, @args) = @_;
 
+    $self->_croak( $ERROR_DONE )
+        if $self->{done};
+    $self->_croak( "Name is required for subcontract" )
+        unless $msg;
+
     my $rep;
     if ( blessed $sub and $sub->isa( "Assert::Refute::Contract" ) ) {
         $rep = $sub->apply(@args);
@@ -239,13 +302,8 @@ sub subcontract {
         $self->_croak("subcontract must be a contract definition or execution log");
     };
 
-    my $stop = !$rep->is_passing;
-    $self->refute( $stop, "$msg (subtest)" );
-    if ($stop) {
-        my $log = $rep->get_log;
-        $self->do_log( $_->[0]+1, $_->[1], $_->[2] )
-            for @$log;
-    };
+    $self->{subcontract}{ $self->get_count + 1 } = $rep;
+    $self->refute( !$rep->is_passing, "$msg (subtest)" );
 };
 
 =head2 QUERYING PRIMITIVES
@@ -326,6 +384,71 @@ sub get_result {
     $self->_croak( "Test $n has never been performed" );
 };
 
+=head3 get_result_details ($id)
+
+Returns a hash containing information about a test:
+
+=over
+
+=item ok - whether the test was successful;
+
+=item name - name of the test.
+
+=item reason - the reason for test failing, if it failed.
+Undefined for "ok" tests.
+
+=item log - any log messages that followed the test (see get_log for format)
+
+=item subcontract - if test was a subcontract, contains the report
+
+=back
+
+Returns empty hash for nonexistent tests, and dies if test number is not integer.
+
+As a special case, test number 0 only contains the log
+for C<diag>/C<note> messages that were written before any tests.
+
+See also L<Test::Tester>.
+
+B<[EXPERIMENTAL]>. Name and meaning may change in the future.
+
+=cut
+
+sub get_result_details {
+    my ($self, $n) = @_;
+
+    $self->_croak( "Bad test number $n, must be nonnegatine integer" )
+        unless defined $n and $n =~ /^[0-9]+$/;
+
+    # Process messages, return if premature(0) or post-mortem (n+1)
+    my @messages;
+    if (my $array = $self->{messages}{$n} ) {
+        @messages = @$array;
+    };
+
+    if ($n < 1 or $n > $self->{count} ) {
+        return { log => \@messages };
+    };
+
+    # now analyze the test itself
+    my $reason = $self->{fail}{$n};
+    my @diag;
+
+    if (ref $reason eq 'ARRAY') {
+        push @diag, [ $self->{indent}, -1, to_scalar($_) ] for @$reason;
+    } elsif ( $reason and $reason ne 1 ) {
+        push @diag, [ $self->{indent}, -1, to_scalar($reason) ];
+    };
+
+    return {
+        ok          => !$reason,
+        name        => $self->{name}{$n},
+        reason      => $reason,
+        log         => [@diag, @messages],
+        subcontract => $self->{subcontract}{$n},
+    };
+};
+
 =head3 get_error
 
 Return last error that was recorded during contract execution,
@@ -365,13 +488,17 @@ B<[NOTE]> that C<diag> is higher than C<ok>.
 =cut
 
 my %padding; # cache level => leading spaces mapping
+my $tab = '    ';
 
 sub get_tap {
     my ($self, $verbosity) = @_;
 
     $verbosity ||= 0;
+
+    my $mess = $self->get_log( $verbosity );
+
     my @str;
-    foreach (@{ $self->{mess} }) {
+    foreach (@$mess) {
         my ($indent, $level, $mess) = @$_;
         next if $level > $verbosity;
 
@@ -382,10 +509,11 @@ sub get_tap {
         $mess    =~ s/\s*$//s;
 
         foreach (split /\n/, $mess) {
-            push @str, "$pad$_\n";
+            push @str, "$pad$_";
         };
     };
-    return join '', @str;
+
+    return join "\n", @str, '';
 };
 
 sub _get_padding {
@@ -506,7 +634,8 @@ sub do_log {
 
     $indent += $self->{indent};
 
-    push @{ $self->{mess} }, [$indent, $level, $mess];
+    $self->{log} ||= $self->{messages}{ $self->{count} } ||= [];
+    push @{ $self->{log} }, [$indent, $level, $mess];
 
     return $self;
 };
@@ -523,43 +652,54 @@ This MAY change in the future.
 =cut
 
 sub get_log {
-    my $self = shift;
-    # TODO copy or smth
-    return $self->{mess};
-};
+    my ($self, $verbosity) = @_;
+    $verbosity = 9**9**9 unless defined $verbosity;
 
-=head3 set_result( $id, $result )
+    my @mess;
 
-Add a refutation to the failed tests log.
+    # output plan if there was plan
+    if (defined $self->{plan_tests}) {
+        push @mess, [ $self->{indent}, 0, "1..$self->{plan_tests}" ]
+            unless $verbosity < 0;
+    };
 
-This is not guaranteed to be called for passing tests.
+    foreach my $n ( 0 .. $self->{count} + 1 ) {
+        # Report test details.
+        # Only append the logs for
+        #   premature (0) and postmortem (count+1) messages
+        if ($n and $n <= $self->{count}) {
+            my $reason = $self->{fail}{$n};
+            my ($level, $prefix)  = $reason ? (-2, "not ok") : (0, "ok");
+            my $name   = $self->{name}{$n} ? "$n - $self->{name}{$n}" : $n;
+            push @mess, [ $self->{indent}, $level, "$prefix $name" ];
 
-=cut
+            if ($self->{subcontract}{$n}) {
+                push @mess, map {
+                    [ $_->[0]+1, $_->[1], $_->[2] ];
+                } @{ $self->{subcontract}{$n}->get_log( $verbosity ) };
+            };
 
-sub set_result {
-    my ($self, $id, $result) = @_;
+            if (ref $reason eq 'ARRAY') {
+                push @mess, map {
+                    [ $self->{indent}, -1, to_scalar( $_ ) ]
+                } @$reason;
+            } elsif ($reason and $reason ne 1) {
+                push @mess, [ $self->{indent}, -1, to_scalar( $reason ) ];
+            };
+        };
 
-    $self->_croak( $ERROR_DONE )
-        if $self->{done};
-    $self->_croak( "Duplicate test id $id" )
-        if exists $self->{fail}{$id};
+        # and all following diags
+        if (my $rest = $self->{messages}{$n} ) {
+            push @mess, grep { $_->[1] <= $verbosity } @$rest;
+        };
+    };
 
-    $self->{fail_count}++ if $result;
-    $self->{fail}{$id} = $result;
+    if (!defined $self->{plan_tests} and $self->{done}) {
+        push @mess, [ $self->{indent}, 0, "1..".$self->get_count ]
+            unless $verbosity < 0;
+    };
 
-    return $self;
-};
-
-=head3 get_proxy
-
-Return ($self, indent) pair in list content, or just $self in scalar context.
-
-=cut
-
-sub get_proxy {
-    my $self = shift;
-
-    return wantarray ? ($self, $self->{indent}) : $self;
+    return \@mess;
 };
 
 sub _croak {
@@ -576,49 +716,20 @@ sub _croak {
 
 =head2 DEPRECATED METHODS
 
-The following methods were added in the beginning and will disappear
-in 0.10.
-
 =over
 
-=item count       => "get_count"
+=item set_result
 
-=item add_result  => "set_result"
-
-=item result      => "get_result"
-
-=item last_error  => "get_error"
-
-=item signature   => "get_sign"
-
-=item as_tap      => "get_tap"
-
-=item log_message => "do_log"
+Was used inside refute() prior to 0.10. This is no more the case.
+This function just dies and will be removed completely in 0.15.
 
 =back
 
 =cut
 
-_deprecate( count       => "get_count" );
-_deprecate( add_result  => "set_result" );
-_deprecate( result      => "get_result" );
-_deprecate( last_error  => "get_error" );
-_deprecate( signature   => "get_sign" );
-_deprecate( as_tap      => "get_tap" );
-_deprecate( log_message => "do_log" );
-
-sub _deprecate {
-    my ($legacy, $new) = @_;
-
-    my $impl = \&$new;
-
-    no strict 'refs';           ## no critic
-    no warnings 'redefine';     ## no critic
-    *$legacy = sub {
-        carp "$legacy() is deprecated, use $new() instead";
-        *$legacy = $impl;
-        goto &$impl;            ## no critic
-    };
+sub set_result {
+    my $self = shift;
+    $self->_croak( "set_result() is removed, use refute() instead" );
 };
 
 =head1 LICENSE AND COPYRIGHT
